# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 23.07.2017 """
from __future__ import unicode_literals
from django.http import HttpResponse
from guardian.models import Permission
from django.contrib.contenttypes.models import ContentType
__author__ = 'mevlanaayas'


# bunların daha user friendly olması için app_label ya da model gibi parametreleri daha açık bir şekilde sun.
def add(name="Can Deneme", codename='deneme', app_label='teamroles', model='Team'):
    content_type = ContentType.objects.get(app_label=app_label.lower(), model=model.lower())
    content_type_id = content_type.id
    new_permission = Permission.objects.create(content_type_id=content_type_id, name=name, codename=codename)
    return HttpResponse("Text only, please.", content_type="text/plain")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from models.role import Role
from models.team import Team
from models.user import User
from models.userrole import UserRole
from models.userteam import UserTeam
from django.contrib import admin

# Register your models here.

admin.site.register(Role)
admin.site.register(Team)
admin.site.register(User)
admin.site.register(UserRole)
admin.site.register(UserTeam)



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 23.07.2017 """
from django.contrib.auth.models import AbstractUser
from django.db import models
from django.utils.translation import ugettext_lazy as _
from teamroles.models import AuditMixin, Team
__author__ = 'mevlanaayas'


class User(AbstractUser, AuditMixin):
    pwd_must_change = models.BooleanField(_('pwd must change'), default=False)

    # 0 -> disabled by another user
    # 1 -> enabled by another user
    # 2 -> disabled due to password expire (won't do until further notice)
    # 3 -> disabled due to many login attempts in given interval
    # 4 -> auto enabled by server if activate user setting is true
    # 5 -> disabled due to not logged in for given interval
    # 6 -> auto enabled when reset password operation completed by disabled user

    disable_reason = models.IntegerField(_("Disable Reason"), blank=True, null=True)

    class Meta(object):
        unique_together = ('email',)
        db_table = 'django_teams_user'

    def team(self):
        team = Team.objects.get(userteam__user=self)
        return team

    def teammates(self):
        team = self.team()
        teammates = User.objects.filter(userteam__team=team)
        return teammates

# bu iki methodla guardianın yaptığı işi daha kolay hale getirmeliyim.
    #  kodlamadan permission eklemek ve kontrol etmek gibi
    #  ve bütün modeller için kullanabilmenin bir yolunu bulsam iyi olabilir

    def perm_exists(self):
        pass

    def add_perm(self):
        pass



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 22.07.2017 """
from django.db import models
from django.utils.translation import ugettext_lazy as _
from teamroles.models import AuditMixin
from teamroles.models.user import User
from teamroles.models.role import Role

__author__ = 'mevlanaayas'


class UserRole(AuditMixin):
    user = models.ForeignKey(User, verbose_name=_('User'), db_column='user')
    role = models.ForeignKey(Role, verbose_name=_('Role'), db_column='role')

    class Meta:
        unique_together = ("user", "role")
        verbose_name = _('User role')
        verbose_name_plural = _('User roles')
        db_table = 'django_teams_user_roles'
        app_label = 'teamroles'

    def __unicode__(self):
        return self.user.username + " has role, " + str(self.role.name)

    def __str__(self):
        return self.user.username + " has role, " + str(self.role.name)



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 23.07.2017 """
from __future__ import unicode_literals, absolute_import
from django.apps import AppConfig
import logging
from django.utils.translation import ugettext_lazy as _

__author__ = 'mevlanaayas'


class TeamrolesConfig(AppConfig):
    """ Common app configuration. """
    name = 'teamroles'
    label = 'teamroles'
    verbose_name = _("Role Base Management Apps for Teams")

    def ready(self):
        logging.debug("Importing %s related services...", self.verbose_name)
        logging.debug("Done!")



from threading import local

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError as ie:
    MiddlewareMixin = object

_tls = local()


class CurrentUserMiddleware(MiddlewareMixin):
    """
    Middleware that handles temporary messages.
    """

    def process_request(self, request):
        """
        Set up TLS' user and request
        """
        _tls.request = request
        _tls.user = getattr(request, 'user', None)

    def process_response(self, request, response):
        """
        Clean TLS after response
        """
        _tls.request = None
        _tls.user = None
        return response

    def process_exception(self, request, exception):
        """
        Clean TLS after exception
        """
        _tls.request = None
        _tls.user = None



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 22.07.2017 """
from django.db import models
from django.utils import timezone
from teamroles.middleware.utils import get_current_user

__author__ = 'mevlanaayas'


class TimeStampMixin(models.Model):
    """
    A mixin to automatically generate created and updated Date attributes for Models
    """
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class AuditMixin(TimeStampMixin):
    """
    AuditMixin will add automatic timestamp of created and modified by who
    """
    created_by = models.CharField(max_length=100, blank=True, editable=False, null=True)
    updated_by = models.CharField(max_length=255, blank=True, editable=False, null=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        user = get_current_user()
        self.updated_at = timezone.now()
        if user:
            self.updated_by = user and user.username or None
        if not self.pk:
            self.created_at = timezone.now()
            self.updated_at = self.created_at
            if user:
                self.created_by = user and user.username or None

        super(AuditMixin, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 22.07.2017 """
from django.db import models
from django.utils.translation import ugettext_lazy as _
from teamroles.models import AuditMixin

__author__ = 'mevlanaayas'


class Team(AuditMixin):
    name = models.CharField(_('Name'), db_column='name', max_length=100)
    code_name = models.CharField(_('Codename'), db_column='codename', max_length=100, unique=True)

    class Meta:
        unique_together = (("name", "code_name"), )
        verbose_name = _('Team')
        verbose_name_plural = _('Teams')
        db_table = 'django_teams_team'
        app_label = 'teamroles'

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name


class AbstractTeam(AuditMixin):
    name = models.CharField(_('Name'), db_column='name', max_length=100)
    code_name = models.CharField(_('Codename'), db_column='codename', max_length=100, unique=True)

    class Meta:
        abstract = True



# method to get user from middleware
def get_current_user():
    from django.conf import settings
    from teamroles.middleware.usermiddleware import _tls
    get_user_fn = getattr(settings, 'GET_CURRENT_USER', None)

    if get_user_fn:
        return get_user_fn()

    return getattr(_tls, 'user', None)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-13 13:35
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.ASCIIUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=30, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, editable=False, max_length=100, null=True)),
                ('updated_by', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('pwd_must_change', models.BooleanField(default=False, verbose_name='pwd must change')),
                ('disable_reason', models.IntegerField(blank=True, null=True, verbose_name='Disable Reason')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'db_table': 'django_teams_user',
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, editable=False, max_length=100, null=True)),
                ('updated_by', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('name', models.CharField(db_column=b'name', max_length=200, unique=True, verbose_name='Role Name')),
                ('status', models.BooleanField(default=True)),
                ('next_role', models.ForeignKey(blank=True, db_column=b'next_role', default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='next_step', to='teamroles.Role', verbose_name='Next Role')),
                ('previous_role', models.ForeignKey(blank=True, db_column=b'previous_role', default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='previous_step', to='teamroles.Role', verbose_name='Previous Role')),
            ],
            options={
                'db_table': 'django_teams_role',
                'verbose_name': 'Role',
                'verbose_name_plural': 'Roles',
            },
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, editable=False, max_length=100, null=True)),
                ('updated_by', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('name', models.CharField(db_column=b'name', max_length=100, verbose_name='Name')),
                ('code_name', models.CharField(db_column=b'codename', max_length=100, unique=True, verbose_name='Codename')),
            ],
            options={
                'db_table': 'django_teams_team',
                'verbose_name': 'Team',
                'verbose_name_plural': 'Teams',
            },
        ),
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, editable=False, max_length=100, null=True)),
                ('updated_by', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('role', models.ForeignKey(db_column=b'role', on_delete=django.db.models.deletion.CASCADE, to='teamroles.Role', verbose_name='Role')),
                ('user', models.ForeignKey(db_column=b'user', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'db_table': 'django_teams_user_roles',
                'verbose_name': 'User role',
                'verbose_name_plural': 'User roles',
            },
        ),
        migrations.CreateModel(
            name='UserTeam',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, editable=False, max_length=100, null=True)),
                ('updated_by', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('team', models.ForeignKey(db_column=b'team', on_delete=django.db.models.deletion.CASCADE, to='teamroles.Team', verbose_name='Team')),
                ('user', models.ForeignKey(db_column=b'user', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'db_table': 'django_teams_user_teams',
                'verbose_name': 'User team',
                'verbose_name_plural': 'User teams',
            },
        ),
        migrations.AlterUniqueTogether(
            name='team',
            unique_together=set([('name', 'code_name')]),
        ),
        migrations.AlterUniqueTogether(
            name='userteam',
            unique_together=set([('user', 'team')]),
        ),
        migrations.AlterUniqueTogether(
            name='userrole',
            unique_together=set([('user', 'role')]),
        ),
        migrations.AlterUniqueTogether(
            name='user',
            unique_together=set([('email',)]),
        ),
    ]



# -*- coding: utf-8 -*-
from distutils.core import setup
setup(
    name='teamroles',
    packages=['teamroles', 'teamroles.models', 'teamroles.middleware', 'teamroles.migrations'],
    version='0.1.8',
    description='Implementation of User-Role-Team-Permissions network with Django web framework',
    author='Mevlana Ayas',
    author_email='mevlanaayas@gmail.com',
    license='MIT',
    url='https://github.com/mevlanaayas/django-teams.git',
    download_url='https://github.com/mevlanaayas/django-teams/tarball/0.1.8',
    keywords=['django', 'django-teams', 'django teams', 'role permissions', 'roles'],
    classifiers=[],
)



# -*- coding: utf-8 -*-
""" Created by Mevlana Ayas on 22/07/2017 """
from django.db import models
from django.utils.translation import ugettext_lazy as _
from teamroles.models.mixins import AuditMixin

__author__ = 'mevlanaayas'


class Role(AuditMixin):
    name = models.CharField(_('Role Name'), db_column='name', max_length=200, unique=True)
    status = models.BooleanField(default=True)
    next_role = models.ForeignKey('self',
                                  verbose_name=_('Next Role'),
                                  db_column='next_role',
                                  related_name='next_step',
                                  null=True,
                                  blank=True,
                                  on_delete=models.SET_NULL,
                                  default=None)
    previous_role = models.ForeignKey('self',
                                      verbose_name=_('Previous Role'),
                                      db_column='previous_role',
                                      related_name='previous_step',
                                      null=True,
                                      blank=True,
                                      on_delete=models.SET_NULL,
                                      default=None)

    class Meta:
        verbose_name = _('Role')
        verbose_name_plural = _('Roles')
        db_table = 'django_teams_role'
        app_label = 'teamroles'

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name


class AbstractRole(AuditMixin):
    name = models.CharField(_('Role Name'), db_column='name', max_length=200, unique=True)
    status = models.BooleanField(default=True)

    class Meta:
        abstract = True
